package de.janschejbal.p2pbluetoothtest.protocol.broadcast;

import java.io.IOException;

import de.janschejbal.p2pbluetoothtest.CONFIG;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolException;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolMessage;
import de.janschejbal.p2pbluetoothtest.protocol.base.ProtocolMessageParser;
import de.janschejbal.p2pbluetoothtest.protocol.base.SafeReader;
import de.janschejbal.p2pbluetoothtest.protocol.base.SafeWriter;

public class HelloMessage implements ProtocolMessage {
	private long nodeId;
	private int  minVersion;
	private int  version;
	private int  connectionId;
	private int  instanceId;
	private long timestamp;
	
	/**
	 * @param nodeId
	 * @param minVersion
	 * @param version
	 * @param connectionId
	 * @param instanceId
	 * @param timestamp
	 */
	public HelloMessage(long nodeId, int minVersion, int version, int connectionId, int instanceId, long timestamp) {
		this.nodeId = nodeId;
		this.minVersion = minVersion;
		this.version = version;
		this.connectionId = connectionId;
		this.instanceId = instanceId;
		this.timestamp = timestamp;
	}

	@Override
	public void send(SafeWriter sink) throws IOException {
		sink.writeHeader(BroadcastProtocolConnection.MESSAGE_HELLO, 8+8+4+4+4+4+8);
		sink.writeLong(CONFIG.BROADCAST_MAGIC);
		sink.writeLong(nodeId);
		sink.writeInt(minVersion);
		sink.writeInt(version);
		sink.writeInt(connectionId);
		sink.writeInt(instanceId);
		sink.writeLong(timestamp);
	}

	private static final ProtocolMessageParser<HelloMessage> parser = new ProtocolMessageParser<HelloMessage>() {
		@Override
		public HelloMessage parseMessage(int tag, int length, SafeReader source) throws IOException {
			if (tag != BroadcastProtocolConnection.MESSAGE_HELLO) throw new ProtocolException("Tag mismatch");
			if (CONFIG.BROADCAST_MAGIC != source.readLong()) throw new ProtocolException("Magic mismatch");
			
			HelloMessage result =  new HelloMessage(
					source.readLong(), // nodeId
					source.readInt(),  // minVersion
					source.readInt(),  // version
					source.readInt(),  // connectionId
					source.readInt(),  // instanceId
					source.readLong()  // timestamp
				);
			source.chompOverhang();
			return result;
		}
	};

	
	
	public static ProtocolMessageParser<HelloMessage> getParser() {
		return parser;
	}
	
	

	public long getNodeId() {
		return nodeId;
	}

	public int getMinVersion() {
		return minVersion;
	}

	public int getVersion() {
		return version;
	}

	public int getConnectionId() {
		return connectionId;
	}

	public int getInstanceId() {
		return instanceId;
	}

	public long getTimestamp() {
		return timestamp;
	}

	public void setNodeId(long nodeId) {
		this.nodeId = nodeId;
	}

	public void setMinVersion(int minVersion) {
		this.minVersion = minVersion;
	}

	public void setVersion(int version) {
		this.version = version;
	}

	public void setConnectionId(int connectionId) {
		this.connectionId = connectionId;
	}

	public void setInstanceId(int instanceId) {
		this.instanceId = instanceId;
	}

	public void setTimestamp(long timestamp) {
		this.timestamp = timestamp;
	}

}
