package de.janschejbal.p2pbluetoothtest;

import java.net.InetSocketAddress;
import java.net.SocketAddress;

public class CONFIG {
	// Time constants
	private static final long SECONDS = 1000;
	private static final long MINUTES = 60 * SECONDS;
	private static final long HOURS   = 60 * MINUTES;
	private static final long DAYS    = 24 * HOURS;
	
	/**
	 * Separate network for debug builds (signed with debug key) and production builds
	 * (signed with production key and possibly pushed to market).
	 * 
	 * Using the wrong setting will be detected and rewarded with a crash. 
	 */
	public static final boolean USE_TESTNET = false;
	
	
	/** if enabled, broadcasts log messages via UDP if connected to WiFi */
	public static final boolean LOGGING_UDP = false;
	public static final SocketAddress UDP_LOG_TARGET = new InetSocketAddress("192.168.50.255", 11111);
	
	/** if enabled, copies of log messages get logged as ADB log messages */
	public static final boolean LOGGING_ADB = USE_TESTNET;
	
	/** Close a log that has been open longer than ... */
	public static final long MAX_LOG_TIME  = 10 * MINUTES;
	/** Close a log that has accumulated more than ... lines */
	public static final int  MAX_LOG_LINES = 10000; // Do not set too small. Re-opening a log creates a few lines, some of them very big
		
	protected static final String ABOUT_URL = "http://mscthesis.janschejbal.de/";
	
	// Upload config
	/** warn if data has been waiting for upload longer than ... */
	public static final long UPLOAD_WARN_AGE = 48 * HOURS;
	/** Wait at most ... between upload attempts (WiFi required for uploads) */
	public static final long MAX_UPLOAD_PAUSE =  30 * MINUTES;
	private static final String SERVER = "https://mscthesis.janschejbal.de/";
	public static final byte[] UPLOAD_HMAC_KEY = "[REDACTED]".getBytes();
	public static final String UPLOAD_TARGET_URL = SERVER + "upload.php";
	public static final String TIME_URL          = SERVER + "time.php";
	public static final String MESSAGING_URL     = SERVER + "msg.php";
	
	
	public final static String BT_NAME_PREFIX = // keep exactly 4 characters long, RegExp-safe characters only
			USE_TESTNET
				? "P2Px"
				: "P2P-";
	public final static String BEACON_PREFIX = "P2P-Beacon-";
	
	
	public final static long CLIENT_CONNECTION_TIMEOUT =  15 * SECONDS;
	public final static long DISCOVERY_TIMEOUT         =  30 * SECONDS;
	public final static long DISCOVERY_INTERVAL        =  60 * SECONDS;
	public final static long COMMUNICATION_TIMEOUT     =  90 * SECONDS;
	
	public final static long BATTERY_LOG_INTERVAL = 5 * MINUTES;
	
	
	public final static int MAX_MESSAGE_LENGTH = 1024 * 1024;

	
	
	public final static long BROADCAST_MAGIC = 
			USE_TESTNET
					? 5778769750148871028L  // "P2Pxtest"
					: 5778769428026323828L; // "P2P-test"
	
	/** minimum version of the broadcast protocol required */
	public final static int BROADCAST_MIN_VERSION = 2;

	public static final long BROADCAST_TIMEOUT = 10 * SECONDS;
	public static final int  BROADCAST_PAYLOAD_SIZE = 1024;

	/** create a new message every .... */
	public static final long BROADCAST_MESSAGE_CREATION_INTERVAL = 10 * MINUTES;
	/** save channel states every ... messages */
	public static final int BROADCAST_SAVE_EVERY = 5;

	/** Do not offer channels that have not gotten a new message in the last ...  */
	public static final long BROADCAST_CHANNEL_EXPIRY_SOFT =   7 * DAYS;
	/** If a channel has not gotten any new messages in the last ... , delete it from storage */
	public static final long BROADCAST_CHANNEL_EXPIRY_HARD =  180 * DAYS; // semi-disable
	// Note: Hard expiry is risky, because it can "resurrect" messages (assuming soft expiry 7 days, hard expiry 15 days):
	//  - t= 0d Node X joins the network, sends a message to A, drops out forever
	//  - t= 6d Node B joins the network 6 days later, gets the message from A
	//  - t= 7d Node A stops propagating the message
	//  - t=10d Node B meets C and gives C the message
	//  - t=13d Node B stops propagating the message	
	//  - t=15d Node A purges the message
	//  - t=16d Node C meets Node A and transmits the entire channel of Node X again <--- !!!!!!!!!!!!!!!
	//  - t=17d Node C stops propagating the message
	//  - t=21d Node B purges the message
	//  - t=22d Node A meets Node B and transmits the entire channel of Node X again <--- !!!!!!!!!!!!!!!
}
